@@ -116,6 +116,8 @@ typedef enum dc_family_t {
 	DC_FAMILY_SPORASUB_SP2 = (18 << 16),
 	/* Deep Six */
 	DC_FAMILY_DEEPSIX_EXCURSION = (19 << 16),
+	/* Seac Screen */
+	DC_FAMILY_SEAC_SCREEN = (20 << 16),
 
 	// Not merged upstream yet
 	/* Garmin */
