@@ -30,8 +30,6 @@
 #include "array.h"
 #include "field-cache.h"
 
-#define C_ARRAY_SIZE(a) (sizeof(a) / sizeof(*(a)))
-
 #define MAXFIELDS 128
 
 struct msg_desc;
@@ -564,7 +562,21 @@ DECLARE_FIELD(DEVICE_INFO, firmware, UINT16)
 }
 
 // SPORT
-DECLARE_FIELD(SPORT, sub_sport, ENUM) { garmin->dive.sub_sport = (ENUM) data; }
+DECLARE_FIELD(SPORT, sub_sport, ENUM) {
+	garmin->dive.sub_sport = (ENUM) data;
+	dc_divemode_t val;
+	switch (data) {
+	case 55: val = DC_DIVEMODE_GAUGE;
+		break;
+	case 56:
+	case 57: val = DC_DIVEMODE_FREEDIVE;
+		break;
+	case 63: val = DC_DIVEMODE_CCR;
+		break;
+	default: val = DC_DIVEMODE_OC;
+	}
+	DC_ASSIGN_FIELD(garmin->cache, DIVEMODE, val);
+}
 
 // DIVE_GAS - uses msg index
 DECLARE_FIELD(DIVE_GAS, helium, UINT8)
@@ -1475,42 +1487,7 @@ garmin_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned i
 {
 	garmin_parser_t *garmin = (garmin_parser_t *) abstract;
 
-	if (!value)
-		return DC_STATUS_INVALIDARGS;
-	if (type == DC_FIELD_TANK_COUNT)
-		type = DC_FIELD_GASMIX_COUNT;
-
-	/* This whole sequence should be standardized */
-	if (!(garmin->cache.initialized & (1 << type)))
-		return DC_STATUS_UNSUPPORTED;
-
-	switch (type) {
-	case DC_FIELD_DIVETIME:
-		return DC_FIELD_VALUE(garmin->cache, value, DIVETIME);
-	case DC_FIELD_MAXDEPTH:
-		return DC_FIELD_VALUE(garmin->cache, value, MAXDEPTH);
-	case DC_FIELD_AVGDEPTH:
-		return DC_FIELD_VALUE(garmin->cache, value, AVGDEPTH);
-	case DC_FIELD_GASMIX_COUNT:
-		return DC_FIELD_VALUE(garmin->cache, value, GASMIX_COUNT);
-	case DC_FIELD_GASMIX:
-		if (flags >= MAXGASES)
-			return DC_STATUS_UNSUPPORTED;
-		return DC_FIELD_INDEX(garmin->cache, value, GASMIX, flags);
-	case DC_FIELD_SALINITY:
-		return DC_FIELD_VALUE(garmin->cache, value, SALINITY);
-	case DC_FIELD_ATMOSPHERIC:
-		return DC_STATUS_UNSUPPORTED;
-	case DC_FIELD_DIVEMODE:
-		return DC_STATUS_UNSUPPORTED;
-	case DC_FIELD_TANK:
-		return DC_STATUS_UNSUPPORTED;
-	case DC_FIELD_STRING:
-		return dc_field_get_string(&garmin->cache, flags, (dc_field_string_t *)value);
-	default:
-		return DC_STATUS_UNSUPPORTED;
-	}
-	return DC_STATUS_SUCCESS;
+	return dc_field_get(&garmin->cache, type, flags, value);
 }
 
 static dc_status_t
