@@ -346,7 +346,7 @@ static const oceanic_common_layout_t oceanic_reactpro_layout = {
 	0xFFF0, /* rb_profile_end */
 	1, /* pt_mode_global */
 	1, /* pt_mode_logbook */
-	1, /* pt_mode_serial */
+	0, /* pt_mode_serial */
 };
 
 static const oceanic_common_layout_t oceanic_proplusx_layout = {
@@ -460,6 +460,7 @@ static const oceanic_common_version_t versions[] = {
 	{"VOYAGE2G \0\0 512K", 0,      &tusa_zenair_layout},
 	{"TUSTALIS \0\0 512K", 0,      &tusa_zenair_layout},
 	{"AMPHOS20 \0\0 512K", 0,      &tusa_zenair_layout},
+	{"AMPAIR20 \0\0 512K", 0,      &tusa_zenair_layout},
 
 	{"REACPRO2 \0\0 512K", 0,      &oceanic_reactpro_layout},
 
@@ -477,6 +478,7 @@ static const oceanic_common_version_t versions[] = {
 	{"AQUA550C \0\0 1024", 0,      &oceanic_oc1_layout},
 	{"WISDOM04 \0\0 1024", 0,      &oceanic_oc1_layout},
 	{"AQUA470C \0\0 1024", 0,      &oceanic_oc1_layout},
+	{"AQUA200C \0\0 1024", 0,      &oceanic_oc1_layout},
 
 	{"OCEANOCI \0\0 1024", 0,      &oceanic_oci_layout},
 
