@@ -61,6 +61,7 @@
 #include "liquivision_lynx.h"
 #include "sporasub_sp2.h"
 #include "deepsix_excursion.h"
+#include "seac_screen.h"
 
 // Not merged upstream yet
 #include "garmin.h"
@@ -107,7 +108,7 @@ dc_parser_new_internal (dc_parser_t **out, dc_context_t *context, dc_family_t fa
 		rc = uwatec_memomouse_parser_create (&parser, context, devtime, systime);
 		break;
 	case DC_FAMILY_UWATEC_SMART:
-		rc = uwatec_smart_parser_create (&parser, context, model, devtime, systime);
+		rc = uwatec_smart_parser_create (&parser, context, model);
 		break;
 	case DC_FAMILY_REEFNET_SENSUS:
 		rc = reefnet_sensus_parser_create (&parser, context, devtime, systime);
@@ -193,6 +194,9 @@ dc_parser_new_internal (dc_parser_t **out, dc_context_t *context, dc_family_t fa
 	case DC_FAMILY_DEEPSIX_EXCURSION:
 		rc = deepsix_excursion_parser_create (&parser, context);
 		break;
+	case DC_FAMILY_SEAC_SCREEN:
+		rc = seac_screen_parser_create (&parser, context);
+		break;
 	default:
 		return DC_STATUS_INVALIDARGS;
 
