@@ -22,9 +22,9 @@
 #include <stdlib.h>
 #include <string.h>	// memcmp
 
-#include <libdivecomputer/reefnet_sensusultra.h>
 #include <libdivecomputer/units.h>
 
+#include "reefnet_sensusultra.h"
 #include "context-private.h"
 #include "parser-private.h"
 #include "array.h"
@@ -132,7 +132,12 @@ reefnet_sensusultra_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *d
 
 	unsigned int timestamp = array_uint32_le (abstract->data + 4);
 
-	dc_ticks_t ticks = parser->systime - (parser->devtime - timestamp);
+	dc_ticks_t ticks = parser->systime;
+	if (timestamp < parser->devtime) {
+		ticks -= parser->devtime - timestamp;
+	} else {
+		ticks += timestamp - parser->devtime;
+	}
 
 	if (!dc_datetime_localtime (datetime, ticks))
 		return DC_STATUS_DATAFORMAT;
